#!/usr/bin/python3.6
"""Create a new Django webapp with a virtualenv.  Defaults to
your free domain, the latest version of Django and Python 3.6

Usage:
  pa_start_django_webapp_with_virtualenv.py [--domain=<domain> --django=<django-version> --python=<python-version>] [--nuke]

Options:
  --domain=<domain>         Domain name, eg www.mydomain.com   [default: your-username.pythonanywhere.com]
  --django=<django-version> Django version, eg "1.8.4"  [default: latest]
  --python=<python-version> Python version, eg "2.7"    [default: 3.6]
  --nuke                    *Irrevocably* delete any existing web app config on this domain. Irrevocably.
"""

from docopt import docopt

from pythonanywhere.snakesay import snakesay
from pythonanywhere.django_project import DjangoProject
from pythonanywhere.utils import ensure_domain


def main(domain, django_version, python_version, nuke):
    domain = ensure_domain(domain)
    project = DjangoProject(domain, python_version)
    project.sanity_checks(nuke=nuke)
    project.create_virtualenv(django_version, nuke=nuke)
    project.run_startproject(nuke=nuke)
    project.find_django_files()
    project.update_settings_file()
    project.run_collectstatic()
    project.create_webapp(nuke=nuke)
    project.add_static_file_mappings()

    project.update_wsgi_file()

    project.webapp.reload()

    print(snakesay(f'All done!  Your site is now live at https://{domain}'))


if __name__ == '__main__':
    arguments = docopt(__doc__)
    main(arguments['--domain'], arguments['--django'], arguments['--python'], nuke=arguments.get('--nuke'))



import warnings

from pythonanywhere.api.base import AuthenticationError, PYTHON_VERSIONS, call_api, get_api_endpoint
from pythonanywhere.api.webapp import Webapp

## TODO PEP 562 __getattr__ should be used here to handle deprecation warnings nicely when we drop python 3.6.
## See https://www.python.org/dev/peps/pep-0562/#id8

warnings.warn(
    """
    Importing from pythonanywhere.api is deprecated in favor of 
    pythonanywhere.api.base for call_api and get_api_endpoint
    and pythonanywhere.api.webapp for Webapp
    """,
    DeprecationWarning
)



#!/usr/bin/python3.6
"""Get list of user's scheduled tasks as a table with columns:
id, interval, at (hour:minute/minute past), status (enabled/disabled), command.

Usage:
  pa_get_scheduled_tasks_list.py [--format TABLEFMT]

Options:
  -h, --help                  Prints this message
  -f, --format TABLEFMT       Sets table format supported by tabulate
                              (defaults to 'simple')

Note:
  This script provides an overview of all tasks. Once a task id is
  known and some specific data is required it's more convenient to get
  it using `pa_get_scheduled_task_specs.py` script instead of parsing
  the table."""

from docopt import docopt
from tabulate import tabulate

from pythonanywhere.scripts_commons import ScriptSchema, get_logger
from pythonanywhere.snakesay import snakesay
from pythonanywhere.task import TaskList


def main(tablefmt):
    logger = get_logger(set_info=True)
    headers = "id", "interval", "at", "status", "command"
    attrs = "task_id", "interval", "printable_time", "enabled", "command"

    def get_right_value(task, attr):
        value = getattr(task, attr)
        if attr == "enabled":
            value = "enabled" if value else "disabled"
        return value

    table = [[get_right_value(task, attr) for attr in attrs] for task in TaskList().tasks]
    msg = tabulate(table, headers, tablefmt=tablefmt) if table else snakesay("No scheduled tasks")
    logger.info(msg)


if __name__ == "__main__":
    schema = ScriptSchema({"--format": ScriptSchema.tabulate_format})
    argument = schema.validate_user_input(docopt(__doc__))

    main(argument.get("format", "simple"))



"""Helpers used by pythonanywhere helper scripts."""

import logging
import sys

from schema import And, Or, Schema, SchemaError, Use

from pythonanywhere.snakesay import snakesay
from pythonanywhere.task import Task

logger = logging.getLogger(__name__)

# fmt: off
tabulate_formats = [
    "plain", "simple", "github", "grid", "fancy_grid", "pipe", "orgtbl", "jira",
    "presto", "psql", "rst", "mediawiki", "moinmoin", "youtrack", "html", "latex",
    "latex_raw", "latex_booktabs", "textile",
]
# fmt: on


class ScriptSchema(Schema):
    """Extends `Schema` adapting it to PA scripts validation strategies.

    Adds predefined schemata as class variables to be used in scripts'
    validation schemas as well as `validate_user_input` method which acts
    as `Schema.validate` but returns a dictionary with converted keys
    ready to be used as function keyword arguments, e.g. validated
    arguments {"--foo": bar, "<baz>": qux} will be be converted to
    {"foo": bar, "baz": qux}. Additional conversion rules may be added as
    dictionary passed to `validate_user_input` :method: as `conversions`
    :param:.

    Use :method:`ScriptSchema.validate_user_input` to obtain kwarg
    dictionary."""

    # class variables are used in task scripts schemata:
    boolean = Or(None, bool)
    hour = Or(None, And(Use(int), lambda h: 0 <= h <= 23), error="--hour has to be in 0..23")
    id_multi = Or([], And(lambda y: [x.isdigit() for x in y], error="<id> has to be integer"))
    id_required = And(Use(int), error="<id> has to be an integer")
    minute_required = And(Use(int), lambda m: 0 <= m <= 59, error="--minute has to be in 0..59")
    minute = Or(None, minute_required)
    string = Or(None, str)
    tabulate_format = Or(
        None,
        And(str, lambda f: f in tabulate_formats),
        error=f"--format should match one of: {', '.join(tabulate_formats)}",
    )

    replacements = {"--": "", "<": "", ">": ""}

    def convert(self, string):
        """Removes cli argument notation characters ('--', '<', '>' etc.).

        :param string: cli argument key to be converted to fit Python
        argument syntax."""

        for key, value in self.replacements.items():
            string = string.replace(key, value)
        return string

    def validate_user_input(self, arguments, *, conversions=None):
        """Calls `Schema.validate` on provided `arguments`.

        Returns dictionary with keys converted by
        `ScriptSchema.convert` :method: to be later used as kwarg
        arguments. Universal rules for conversion are stored in
        `replacements` class variable and may be updated using
        `conversions` kwarg. Use optional `conversions` :param: to add
        custom replacement rules.

        :param arguments: dictionary of cli arguments provided be
        (e.g.) `docopt`
        :param conversions: dictionary of additional rules to
        `self.replacements`"""

        if conversions:
            self.replacements.update(conversions)

        try:
            self.validate(arguments)
            return {self.convert(key): val for key, val in arguments.items()}
        except SchemaError as e:
            logger.warning(snakesay(str(e)))
            sys.exit(1)


def get_logger(set_info=False):
    """Sets logger for 'pythonanywhere' package.

    Returns `logging.Logger` instance with no message formatting which
    will stream to stdout. With `set_info` :param: set to `True`
    logger defines `logging.INFO` level otherwise it leaves default
    `logging.WARNING`.

    To toggle message visibility in scripts use `logger.info` calls
    and switch `set_info` value accordingly.

    :param set_info: boolean (defaults to False)"""

    logging.basicConfig(format="%(message)s", stream=sys.stdout)
    logger = logging.getLogger("pythonanywhere")
    if set_info:
        logger.setLevel(logging.INFO)
    else:
        logger.setLevel(logging.WARNING)
    return logger


def get_task_from_id(task_id, no_exit=False):
    """Get `Task.from_id` instance representing existing task.

    :param task_id: integer (should be a valid task id)
    :param no_exit: if (default) False sys.exit will be called when
      exception is caught"""

    try:
        return Task.from_id(task_id)
    except Exception as e:
        logger.warning(snakesay(str(e)))
        if not no_exit:
            sys.exit(1)



from pathlib import Path
import uuid

from pythonanywhere.api.webapp import Webapp
from pythonanywhere.exceptions import SanityException
from pythonanywhere.virtualenvs import Virtualenv
from pythonanywhere.launch_bash_in_virtualenv import launch_bash_in_virtualenv
from pythonanywhere.snakesay import snakesay


class Project:
    def __init__(self, domain, python_version):
        self.domain = domain
        self.python_version = python_version
        self.project_path = Path(f'~/{domain}').expanduser()
        self.virtualenv = Virtualenv(self.domain, self.python_version)
        self.wsgi_file_path = Path(
            f"/var/www/{domain.replace('.', '_')}_wsgi.py"
        )
        self.webapp = Webapp(domain)


    def sanity_checks(self, nuke):
        self.webapp.sanity_checks(nuke=nuke)
        if nuke:
            return
        if self.virtualenv.path.exists():
            raise SanityException(
                "You already have a virtualenv for {domain}.\n\n"
                "Use the --nuke option if you want to replace it.".format(
                    domain=self.domain
                )
            )
        if self.project_path.exists():
            raise SanityException(
                "You already have a project folder at {project_path}.\n\n"
                "Use the --nuke option if you want to replace it.".format(
                    project_path=self.project_path
                )
            )


    def create_webapp(self, nuke):
        self.webapp.create(self.python_version, self.virtualenv.path, self.project_path, nuke=nuke)


    def add_static_file_mappings(self):
        self.webapp.add_default_static_files_mappings(self.project_path)


    def start_bash(self):
        print(snakesay('Starting Bash shell with activated virtualenv in project directory.  Press Ctrl+D to exit.'))
        unique_id = str(uuid.uuid4())
        launch_bash_in_virtualenv(self.virtualenv.path, unique_id, self.project_path)



#!/usr/bin/python3.6
"""Set the HTTPS certificate and private key for a website to the contents of two files, and reload the site.

Usage:
  pa_set_webapp_ssl.py <domain> <certificate-file> <private-key-file> [--suppress-reload]

Options:
  <domain>              Domain name, eg www.mydomain.com
  <certificate-file>    The name of the file containing the combined certificate in PEM format (normally
                        a number of blocks, each one starting "BEGIN CERTIFICATE" and ending "END CERTIFICATE")
  <private-key-file>    The name of the file containing the private key in PEM format (a file with one block,
                        starting with something like "BEGIN PRIVATE KEY" and ending with something like
                        "END PRIVATE KEY")
  --suppress-reload     The website will need to be reloaded in order to activate the new certificate/key combination
                        -- this happens by default, use this option to suppress it.
"""

from docopt import docopt
import os
import sys

from pythonanywhere.api.webapp import Webapp
from pythonanywhere.snakesay import snakesay


def main(domain_name, certificate_file, private_key_file, suppress_reload):
    if not os.path.exists(certificate_file):
        print(f"Could not find certificate file {certificate_file}")
        sys.exit(1)
    with open(certificate_file, "r") as f:
        certificate = f.read()

    if not os.path.exists(private_key_file):
        print(f"Could not find private key file {private_key_file}")
        sys.exit(1)
    with open(private_key_file, "r") as f:
        private_key = f.read()

    webapp = Webapp(domain_name)
    webapp.set_ssl(certificate, private_key)
    if not suppress_reload:
        webapp.reload()

    ssl_details = webapp.get_ssl_info()
    print(snakesay(
        "That's all set up now :-)\n"
        "Your new certificate will expire on {expiry:%d %B %Y},\n"
        "so shortly before then you should renew it\n"
        "and install the new certificate.".format(
            expiry=ssl_details["not_after"]
        )
    ))


if __name__ == '__main__':
    arguments = docopt(__doc__)
    main(
        arguments['<domain>'],
        arguments['<certificate-file>'], arguments['<private-key-file>'],
        suppress_reload=arguments.get('--suppress-reload')
    )



import os

import requests

PYTHON_VERSIONS = {
    "3.6": "python36",
    "3.7": "python37",
    "3.8": "python38",
    "3.9": "python39",
    "3.10": "python310",
}


class AuthenticationError(Exception):
    pass


class NoTokenError(Exception):
    pass


def get_api_endpoint():
    hostname = os.environ.get(
        "PYTHONANYWHERE_SITE",
        "www." + os.environ.get(
            "PYTHONANYWHERE_DOMAIN",
            "pythonanywhere.com"
        )
    )
    return f"https://{hostname}/api/v0/user/{{username}}/{{flavor}}/"


def call_api(url, method, **kwargs):
    token = os.environ.get("API_TOKEN")
    if token is None:
        raise NoTokenError(
            "Oops, you don't seem to have an API token.  "
            "Please go to the 'Account' page on PythonAnywhere, then to the 'API Token' "
            "tab.  Click the 'Create a new API token' button to create the token, then "
            "start a new console and try running this script again."
        )
    insecure = os.environ.get("PYTHONANYWHERE_INSECURE_API") == "true"
    response = requests.request(
        method=method,
        url=url,
        headers={"Authorization": f"Token {token}"},
        verify=not insecure,
        **kwargs
    )
    if response.status_code == 401:
        print(response, response.text)
        raise AuthenticationError(
            f"Authentication error {response.status_code} calling API: {response.text}"
        )
    return response



from pathlib import Path
import re
import shutil
import subprocess

from packaging import version

from pythonanywhere.exceptions import SanityException
from pythonanywhere.snakesay import snakesay
from .project import Project


class DjangoProject(Project):
    def django_version_newer_or_equal_than(self, this_version):
        return version.parse(self.virtualenv.get_version("django")) >= version.parse(this_version)

    def download_repo(self, repo, nuke):
        if nuke and self.project_path.exists():
            shutil.rmtree(str(self.project_path))
        subprocess.check_call(['git', 'clone', repo, str(self.project_path)])

    def ensure_branch(self, branch):
        output = subprocess.check_output(
            ["git", "-C", str(self.project_path), "branch", "-r"]
        ).decode().rstrip().split("\n")
        branches = [x.strip().replace("origin/", "") for x in output if "->" not in x]
        if branch == "None" and len(branches) == 1:
            return
        if branch == "None":
            shutil.rmtree(str(self.project_path))
            raise SanityException(
                "There are many branches in your repo. "
                "You need to specify which branch to use by adding "
                "--branch=<branch> option to the command."
            )
        if branch not in branches:
            shutil.rmtree(str(self.project_path))
            raise SanityException(f"You do not have a {branch} branch in your repo")
        #
        current_branch = subprocess.check_output(
            ["git", "-C", str(self.project_path), "rev-parse", "--abbrev-ref HEAD"]
        ).decode().strip()

        if current_branch != branch:
            subprocess.check_call(["git", "-C", str(self.project_path), "checkout", branch])

    def create_virtualenv(self, django_version=None, nuke=False):
        self.virtualenv.create(nuke=nuke)
        if django_version is None:
            packages = self.detect_requirements()
        elif django_version == 'latest':
            packages = 'django'
        else:
            packages = f'django=={django_version}'
        self.virtualenv.pip_install(packages)

    def detect_requirements(self):
        requirements_txt = self.project_path / 'requirements.txt'
        if requirements_txt.exists():
            return f'-r {requirements_txt.resolve()}'
        return 'django'

    def run_startproject(self, nuke):
        print(snakesay('Starting Django project'))
        if nuke and self.project_path.exists():
            shutil.rmtree(str(self.project_path))
        self.project_path.mkdir()

        new_django = self.django_version_newer_or_equal_than("4.0")
        django_admin_executable = "django-admin" if new_django else "django-admin.py"

        subprocess.check_call([
            str(Path(self.virtualenv.path) / "bin" / django_admin_executable),
            "startproject",
            "mysite",
            str(self.project_path),
        ])


    def find_django_files(self):
        try:
            self.settings_path = next(self.project_path.glob('**/settings.py'))
        except StopIteration:
            raise SanityException('Could not find your settings.py')
        try:
            self.manage_py_path = next(self.project_path.glob('**/manage.py'))
        except StopIteration:
            raise SanityException('Could not find your manage.py')


    def update_settings_file(self):
        print(snakesay('Updating settings.py'))

        with self.settings_path.open() as f:
            settings = f.read()
        new_settings = settings.replace(
            'ALLOWED_HOSTS = []',
            f'ALLOWED_HOSTS = [{self.domain!r}]'
        )

        new_django = self.django_version_newer_or_equal_than("3.1")

        if re.search(r'^MEDIA_ROOT\s*=', settings, flags=re.MULTILINE) is None:
            new_settings += "\nMEDIA_URL = '/media/'"
        if re.search(r'^STATIC_ROOT\s*=', settings, flags=re.MULTILINE) is None:
            if new_django:
                new_settings += "\nSTATIC_ROOT = Path(BASE_DIR / 'static')"
            else:
                new_settings += "\nSTATIC_ROOT = os.path.join(BASE_DIR, 'static')"
        if re.search(r'^MEDIA_ROOT\s*=', settings, flags=re.MULTILINE) is None:
            if new_django:
                new_settings += "\nMEDIA_ROOT = Path(BASE_DIR / 'media')"
            else:
                new_settings += "\nMEDIA_ROOT = os.path.join(BASE_DIR, 'media')"

        with self.settings_path.open('w') as f:
            f.write(new_settings)


    def run_collectstatic(self):
        print(snakesay('Running collectstatic'))
        subprocess.check_call([
            str(Path(self.virtualenv.path) / 'bin/python'),
            str(self.manage_py_path),
            'collectstatic',
            '--noinput',
        ])


    def run_migrate(self):
        print(snakesay('Running migrate database'))
        subprocess.check_call([
            str(Path(self.virtualenv.path) / 'bin/python'),
            str(self.manage_py_path),
            'migrate',
        ])


    def update_wsgi_file(self):
        print(snakesay(f'Updating wsgi file at {self.wsgi_file_path}'))
        template = (Path(__file__).parent / 'wsgi_file_template.py').open().read()
        with self.wsgi_file_path.open('w') as f:
            f.write(template.format(project=self))



from pathlib import Path
from setuptools import setup

here = Path(__file__).parent

# Get the long description from the README file
with (here / "README.md").open(encoding="utf-8") as f:
    long_description = f.read()

setup(
    name="pythonanywhere",
    version="0.10.2",
    description="PythonAnywhere helper tools for users",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/pythonanywhere/helper_scripts/",
    author="PythonAnywhere LLP",
    author_email="developers@pythonanywhere.com",
    license="MIT",
    classifiers=[
        "Development Status :: 3 - Alpha",
        "Intended Audience :: Developers",
        "Topic :: Software Development :: Libraries",
        "License :: OSI Approved :: MIT License",
        "Programming Language :: Python :: 3.10",
        "Programming Language :: Python :: 3.9",
        "Programming Language :: Python :: 3.8",
        "Programming Language :: Python :: 3.7",
        "Programming Language :: Python :: 3.6",
    ],
    keywords="pythonanywhere api cloud web hosting",
    packages=["cli", "pythonanywhere", "pythonanywhere.api"],
    install_requires=[
        "docopt",
        "packaging",
        "python-dateutil",
        "requests",
        "schema",
        "tabulate",
        "typer",
    ],
    extras_require={},
    python_requires=">=3.6",
    package_data={},
    data_files=[],
    entry_points={},
    scripts=[
        "cli/pa",
        "pythonanywhere/snakesay.py",
        "scripts/pa_autoconfigure_django.py",
        "scripts/pa_create_scheduled_task.py",
        "scripts/pa_create_webapp_with_virtualenv.py",
        "scripts/pa_delete_scheduled_task.py",
        "scripts/pa_delete_webapp_logs.py",
        "scripts/pa_get_scheduled_task_specs.py",
        "scripts/pa_get_scheduled_tasks_list.py",
        "scripts/pa_install_webapp_letsencrypt_ssl.py",
        "scripts/pa_install_webapp_ssl.py",
        "scripts/pa_reload_webapp.py",
        "scripts/pa_start_django_webapp_with_virtualenv.py",
        "scripts/pa_update_scheduled_task.py",
    ],
)



"""User interface for interacting with PythonAnywhere files.
Provides a class `PAPath` which should be used by helper scripts
providing features for programmatic handling of user's files."""

import getpass
import logging
from urllib.parse import urljoin

from pythonanywhere.api.files_api import Files
from pythonanywhere.snakesay import snakesay

logger = logging.getLogger("pythonanywhere")


class PAPath:
    """Class providing interface for interacting with PythonAnywhere
    user files.

    Class should be instantiated with a path to an existing
    PythonAnywhere file or directory that user has access to or to an
    available destination path for a file that would be uploaded.

    To get PythonAnywhere url for given path use
    :property:`PAPath.url`, to get its contents use
    :property:`PAPath.contents` or :property:`PAPath.tree` for a list
    of regular paths, when given path is directory.

    To perform actions on path pointing to an existing PythonAnywhere
    file/directory, use following methods:
    - :method:`PAPath.delete` to delete file/directory
    - :method:`PAPath.upload` to overwrite file contents
    - :method:`PAPath.share` to start sharing a file
    - :method:`PAPath.unshare` to stop sharing a file
    - :method:`PAPath.get_sharing_url` to check if file is already
    shared and get its sharing url

    When path does not represent existing PythonAnywhere file, it can
    be created with :method:`PAPath.upload`."""

    def __init__(self, path):
        self.path = self._standarize_path(path)
        self.api = Files()

    def __repr__(self):
        return self.url

    def _make_sharing_url(self, path):
        return urljoin(self.api.base_url.split("api")[0], path)

    @staticmethod
    def _standarize_path(path):
        return path.replace("~", f"/home/{getpass.getuser()}") if path.startswith("~") else path

    @property
    def url(self):
        """Returns url to PythonAnywhere for `self.path`.  Does not
        perform any checks (url might not point to an existing file)."""

        files_base = self.api.base_url.replace("/api/v0", "")
        return f"{files_base[:-1]}{self.path}"

    @property
    def contents(self):
        """When `self.path` points to a PythonAnywhere user
        directiory, returns a dictionary of its files and directories,
        where file/directory names are keys and values contain
        information about type and API endpoint.  Otherwise (when
        `self.path` points to a file) contents of the file are
        returned as bytes.

        >>> PAPath('/home/username').contents
        >>> {'.bashrc': {'type': 'file',
            'url': 'https://www.pythonanywhere.com/api/v0/user/username/files/path/home/username/.bashrc'},
            '.local': {'type': 'directory',
            'url': 'https://www.pythonanywhere.com/api/v0/user/username/files/path/home/username/.local'},
            ... }

        >>> PAPath('/home/username/README.txt').contents
        >>> b"some README.txt contents..."
        """

        try:
            content = self.api.path_get(self.path)
            return content if isinstance(content, dict) else content.decode("utf-8")
        except Exception as e:
            logger.warning(snakesay(str(e)))
            return None

    @property
    def tree(self):
        """Returns list of regular directories and files for
        `self.path`.  'Regular' means non dotfiles nor symlinks.
        Result is trimmed to 1000 items.

        >>> PAPath('/home/username').tree
        >>> ['/home/username/README.txt']
        """

        try:
            return self.api.tree_get(self.path)
        except Exception as e:
            logger.warning(snakesay(str(e)))
            return None

    def delete(self):
        """Returns `True` when `self.path` successfully deleted on
        PythonAnywhere, `False` otherwise."""

        try:
            self.api.path_delete(self.path)
            logger.info(snakesay(f"{self.path} deleted!"))
            return True
        except Exception as e:
            logger.warning(snakesay(str(e)))
            return False

    def upload(self, content):
        """Returns `True` when provided `content` successfully
        uploaded to `self.path`.  If `self.path` already existed on
        PythonAnywhere, it will be overwritten by the `content`.
        When upload is not successful, returns `False`."""

        try:
            result = self.api.path_post(self.path, content)
        except Exception as e:
            logger.warning(snakesay(str(e)))
            return False

        msg = {
            200: f"{self.path} successfully updated!",
            201: f"Content successfully uploaded to {self.path}!"
        }[result]

        logger.info(snakesay(msg))
        return True

    def get_sharing_url(self, quiet=False):
        """Returns PythonAnywhere sharing url for `self.path` if file
        is shared, empty string otherwise."""

        url = self.api.sharing_get(self.path)
        if url:
            sharing_url = self._make_sharing_url(url)
            if not quiet:
                logger.info(snakesay(f"{self.path} is shared at {sharing_url}"))
            return sharing_url

        logger.info(snakesay(f"{self.path} has not been shared"))

        return ""

    def share(self):
        """Returns PythonAnywhere sharing link for `self.path` or an
        empty string when share not successful."""

        try:
            code, url = self.api.sharing_post(self.path)
        except Exception as e:
            logger.warning(snakesay(str(e)))
            return ""

        msg = {200: "was already", 201: "successfully"}[code]
        sharing_url = self._make_sharing_url(url)
        logger.info(snakesay(f"{self.path} {msg} shared at {sharing_url}"))
        return sharing_url

    def unshare(self):
        """Returns `True` when file unshared or has not been shared,
        `False` otherwise."""

        already_shared = self.get_sharing_url(quiet=True)
        if already_shared:
            result = self.api.sharing_delete(self.path)
            if result == 204:
                logger.info(snakesay(f"{self.path} is no longer shared!"))
                return True
            logger.warning(snakesay(f"Could not unshare {self.path}... :("))
            return False
        logger.info(snakesay(f"{self.path} is not being shared, no need to stop sharing..."))
        return True
